import json
import os
from typing import Any

import attr

from config.cloudfoundry import (
    default_vcap_application,
    default_vcap_services
)

from glom import glom


@attr.s(slots=True)
class CFenv:
    vcap_service_prefix = attr.ib(
        type=str,
        default=os.getenv('VCAP_SERVICE_PREFIX', 'p.config-server'),
        validator=attr.validators.instance_of(str)
    )
    vcap_application = attr.ib(
        type=dict,
        default=json.loads(
            os.getenv('VCAP_APPLICATION', default_vcap_application)
        ),
        validator=attr.validators.instance_of(dict)
    )
    vcap_services = attr.ib(
        type=dict,
        default=json.loads(
            os.getenv('VCAP_SERVICES', default_vcap_services)
        ),
        validator=attr.validators.instance_of(dict)
    )

    def __attrs_post_init__(self) -> None:
        if self.vcap_service_prefix not in self.vcap_services.keys():
            vcap_services_copy = self.vcap_services.copy()
            vcap_services_copy[self.vcap_service_prefix] = vcap_services_copy.pop('p.config-server')
            self.vcap_services = vcap_services_copy

    @property
    def space_name(self) -> Any:
        return glom(self.vcap_application, 'space_name', default='')

    @property
    def organization_name(self) -> Any:
        return glom(self.vcap_application, 'organization_name', default='')

    @property
    def application_name(self) -> Any:
        return glom(self.vcap_application, 'application_name', default='')

    @property
    def uris(self) -> Any:
        return glom(self.vcap_application, 'uris', default=[])

    def configserver_uri(self, vcap_path: str = '.0.credentials.uri') -> Any:
        path = self._format_vcap_path(vcap_path)
        return glom(self.vcap_services, path, default='')

    def configserver_access_token_uri(self, vcap_path: str = '.0.credentials.access_token_uri') -> Any:
        path = self._format_vcap_path(vcap_path)
        return glom(self.vcap_services, path, default='')

    def configserver_client_id(self, vcap_path: str = '.0.credentials.client_id') -> Any:
        path = self._format_vcap_path(vcap_path)
        return glom(self.vcap_services, path, default='')

    def configserver_client_secret(self, vcap_path: str = '.0.credentials.client_secret') -> Any:
        path = self._format_vcap_path(vcap_path)
        return glom(self.vcap_services, path, default='')

    def _format_vcap_path(self, path) -> str:
        return f"{self.vcap_service_prefix}{path}"



import json


default_vcap_services = json.dumps(
    {
        "p.config-server": [
            {
                "credentials": {
                    "credhub-ref": ""
                }
            }
        ]
    }
)
default_vcap_application = json.dumps(
    {
        "application_name": "",
        "space_name": "",
        "organization_name": "",
        "uris": []
    }
)



from collections import OrderedDict

import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="config-client3-pcf",
    version="0.0.1",
    author="alexandre menezes/jeff m",
    author_email="quicksh0t12@gmail.com",
    description="config service client for pcf config-server3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='Apache-2.0',
    url="https://github.com/thedonutz/config-client",
    packages=setuptools.find_packages(),
    python_requires='>=3.6.0',
    project_urls=OrderedDict((
        ('Documentation', 'https://github.com/thedonutz/config-client'),
        ('Code', 'https://github.com/thedonutz/config-client'),
        ('Issue tracker', 'https://github.com/thedonutz/config-client/issues')
    )),
    install_requires=[
        'attrs>=19.1.0',
        'glom>=19.2.0',
        'requests>=2.22.0'
    ],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: AsyncIO",
        "Framework :: Flask",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Java Libraries",
    ],
)



"""Module for retrieve application's config from Spring Cloud Config."""
import logging
import os
import sys
from typing import Any, Callable, Dict, KeysView

import attr

from config.core import singleton

from glom import glom

import requests


logging.getLogger(__name__).addHandler(logging.NullHandler())


@attr.s(slots=True)
class ConfigClient:
    """ConfigClient client."""

    address = attr.ib(
        type=str, default=os.getenv("CONFIGSERVER_ADDRESS", "http://localhost:8888")
    )
    branch = attr.ib(type=str, default=os.getenv("BRANCH", "master"))
    app_name = attr.ib(
        type=str,
        default=os.getenv("APP_NAME", ""),
        validator=attr.validators.instance_of(str),
    )
    profile = attr.ib(type=str, default=os.getenv("PROFILE", "development"))
    url = attr.ib(
        type=str,
        default="{address}/{branch}/{app_name}-{profile}.json"
    )
    _config = attr.ib(
        type=dict, default={}, init=False, validator=attr.validators.instance_of(dict)
    )

    def __attrs_post_init__(self) -> None:
        """Format ConfigClient URL."""
        self.url = self.url.format(
            address=self.address,
            app_name=self.app_name,
            branch=self.branch,
            profile=self.profile,
        )
        self._ensure_request_json()

    def _ensure_request_json(self) -> None:
        if not self.url.endswith(".json"):
            dot_position = self.url.rfind(".")
            if dot_position > 0:
                self.url = self.url.replace(self.url[dot_position:], ".json")
            else:
                self.url = f"{self.url}.json"
            logging.warning(
                "URL suffix adjusted to a supported format. "
                "For more details see: "
                "https://github.com/amenezes/config-client/#default-values"
            )
        logging.debug(f"Target URL configured: {self.url}")

    def get_config(self, headers: Dict = {}) -> None:
        """Retrieve configuration from Spring ConfigClient."""
        try:
            logging.debug(f"Requesting: {self.url}")

            response = requests.get(self.url, headers=headers)
            logging.debug(f"HTTP response code: {response.status_code}")
            if response.ok:
                self._config = response.json()
            else:
                raise Exception(
                    "Failed to retrieve the configurations. "
                    f"HTTP Response code: {response.status_code}."
                )

        except Exception:
            logging.error("Failed to establish connection with ConfigServer.")
            sys.exit(1)

    @property
    def config(self) -> Dict:
        """Getter from configurations retrieved from ConfigClient."""
        return self._config

    def get_attribute(self, value: str) -> Any:
        """Get attribute from configurations."""
        return glom(self._config, value, default="")

    def get_keys(self) -> KeysView:
        """List all keys from configuration retrieved."""
        return self._config.keys()


@singleton
def create_config_client(**kwargs):
    obj = ConfigClient(**kwargs)
    obj.get_config()
    return obj


def config_client(*args, **kwargs) -> Callable[[Dict[str, str]], ConfigClient]:
    logging.debug(f"args: {args}")
    logging.debug(f"kwargs: {kwargs}")

    def wrap_function(function):
        logging.debug(f"caller: {function}")

        def enable_config():
            obj = ConfigClient(*args, **kwargs)
            obj.get_config()
            return function(obj)

        return enable_config

    return wrap_function
